#!/usr/bin/perl
# -*- cperl -*-

# This file is part of MADDE
#
# Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
#
# Contact: Juha Riihimäki <juha.riihimaki@nokia.com>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# version 2.1 as published by the Free Software Foundation.
#
# This library is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
# 02110-1301 USA

# this script is used in the case where /etc/network/interfaces
# is used by system to configure usb0 networking (too).
# alternative would be that usb0 is configured outside of that.

use strict;
use warnings;

sub usage($)
{
    die "Usage: $0 $_[0]\n";
}

usage 'command <args>' unless defined $ARGV[0];

sub xopenI($)
{
    open I, '<', $_[0] or die "Cannot open '$_[0]' for reading: $!.\n";
}
sub xopenO($)
{
    open O, '>', $_[0] or die "Cannot open '$_[0]' for writing: $!.\n";
}

# when testing . make these './etc/...'
my $conffile = '/etc/developer-mode.conf';
my $netifile = '/etc/network/interfaces';

my %config = ( usb0_mode => '', usb0_prevmode => '',
	       usb0_address => '', usb0_netmask => '',
	       usb0_gateway => '', usb0_peerip => '');

sub read_config()
{
    xopenI $conffile;
    while (<I>) {
	$config{$1} = $2 if /^cnf_(\w+)='(.*?)'\s*$/;
    }
}

sub rotate_conffile($)
{
    unlink $_[0] . '.old';
    rename $_[0], $_[0] . '.old';
    rename $_[0] . '.new', $_[0];
}

sub modify_interfaces($)
{
    xopenI $netifile;
    xopenO $netifile . '.new';
    system 'ifdown', 'usb0'; # opportunistic

    my $mode = $_[0];
    $config{'usb0_prevmode'} = $config{'usb0_mode'}
      unless $config{'usb0_mode'} eq 'manual';
    $config{'usb0_mode'} = $mode;

    while (<I>) {
	last if /^\s*iface\s+usb0\b/;
	print O $_;
    }
    print O "\n" if eof I;

    if ($mode eq 'dhcp' or $mode eq 'manual') {
	print O "iface usb0 inet $mode\n";
    }
    else { # static || dhcpd
	print O "iface usb0 inet static\n";
	print O "\taddress $config{usb0_address}\n";
	print O "\tnetmask $config{usb0_netmask}\n";
	print O "\tgateway $config{usb0_gateway}\n"
	  if length $config{usb0_gateway};
    }
    if ($mode eq 'dhcpd') {
	my $peerip = $config{usb0_peerip};
	print O "\tup /usr/lib/sdk-connectivity-tool/usbdhcpd.sh $peerip\n"; #changed path from /usr/lib/sdkconnectivitytool to sdk-connectivity-tool
	print O "\tdown /usr/lib/sdk-connectivity-tool/usbdhcpd.sh stop\n"; #changed path from /usr/lib/sdkconnectivitytool to sdk-connectivity-tool
    }
    print O "\n";
    my @xlist = ();
    while (<I>) {
	push @xlist, $_ if /^\s*$/ or /^\s*#/;
	last if /^\s*(iface|mapping|auto)\b/ or /^\s*allow-/;
    }
    foreach (@xlist) { $_ = '', last if /^\s*$/; }
    print O @xlist;
    print O $_ while (<I>);
    close O;
    close I;
    rotate_conffile $netifile;
    system 'ifup', 'usb0' unless $mode eq 'manual'; # opportunistic
}

sub update_config()
{
    xopenI $conffile;
    xopenO $conffile . '.new';
    while (<I>) {
	if (/^cnf_(\w+)\s*=\s*(.*?)\s*/)
	{
	    my $val = $config{$1} || '';
	    print O 'cnf_', $1, "='", $val, "'\n" if length $val;
	    delete $config{$1};
	}
	else { print O $_; }
    }
    close I;
    while ( my ($key, $value) = each %config) {
	print O 'cnf_', $key, "='", $value, "'\n" if length $value;
    }
    close O;
    rotate_conffile $conffile;
}

if ($ARGV[0] eq 'configure-usb0')
{
    read_config;

    usage 'configure-usb0 <mode> [address netmask [gateway]|peerip]'
      unless defined $ARGV[1];

    my $mode = $ARGV[1];

    if ($mode eq 'previous') {
	modify_interfaces $config{'usb0_prevmode'};
	update_config;
	exit 0;
    }
    # else
    if ($mode eq 'dhcp' or $mode eq 'manual') {
	exit 0 if ($config{'usb0_mode'} eq $mode); # no change

	modify_interfaces $mode;
	update_config;
	exit 0;
    }
    # else, static (+ dhcpd)configuration.
    my $dhcpd = $mode eq 'dhcpd'? 1: 0;
    my ($peerip, $gateway) = ('', '');
    if ($dhcpd) {
	usage 'configure-usb0 <address> <netmask> <peerip>'
	  unless defined $ARGV[4];
	$peerip = $ARGV[4];
    }
    else {
	usage 'configure-usb0 <address> <netmask> [gateway]'
	  unless defined $ARGV[3];
	$peerip = $config{'usb0_peerip'};
	$gateway = $ARGV[4] if defined $ARGV[4];
    }
    # XXX check ip formats.
    if ($config{'usb0_mode'} eq $mode and
	$config{'usb0_address'} eq $ARGV[2] and
	$config{'usb0_netmask'} eq $ARGV[3] and
	$config{'usb0_peerip'} eq $peerip and
	$config{'usb0_gateway'} eq $gateway) { exit 0; }

    $config{'usb0_address'} = $ARGV[2]; $config{'usb0_netmask'} = $ARGV[3];
    $config{'usb0_gateway'} = $gateway; $config{'usb0_peerip'} = $peerip;
    modify_interfaces $mode;
    update_config;
    exit 0;
}
usage 'command <args>';

__END__

test cases:

  perl usb0.pl configure-usb0 dhcp
  perl usb0.pl configure-usb0 static 192.168.2.15 255.255.255.0
  perl usb0.pl configure-usb0 dhcp 192.168.2.15 255.255.255.0
  perl usb0.pl configure-usb0 static 192.168.2.15 255.255.255.0
  perl usb0.pl configure-usb0 static 192.168.2.15 255.255.255.1
  perl usb0.pl configure-usb0 static 192.168.2.15 255.255.255.0
  perl usb0.pl configure-usb0 dhcpd 192.168.2.15 255.255.255.0 192.168.2.14
  perl usb0.pl configure-usb0 manual
  perl usb0.pl configure-usb0 previous
